/**
 * CarManagerSettings.java
 *
 * Copyright (C) 2005-2006 Tommi Laukkanen
 * http://www.substanceofcode.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package com.substanceofcode.carmanager.model;

import javax.microedition.midlet.MIDlet;

/**
 *
 * @author Tommi
 */
public class CarManagerSettings {
    
    /** Currency symbol */
    private String m_currencySymbol;
    
    /** Entry type */
    private int m_entryType;
    private final static int ENTRYTYPE_FULLFILLUP = 0;
    private final static int ENTRYTYPE_PRICEPERLITRE = 1;
    
    /** Settings */
    private static Settings m_settings;
    private static CarManagerSettings m_singleton;
    
    /** Creates a new instance of CarManagerSettings */
    private CarManagerSettings(String currencySymbol, int entryType) {
        m_currencySymbol = currencySymbol;
        m_entryType = entryType;
    }
    
    /** Get a singleton instance of this class */
    public static CarManagerSettings getInstance(MIDlet midlet) {
        /** Open settings */
        if(m_settings==null) {
            try{
                m_settings = Settings.getInstance(midlet);
            }catch(Exception ex) {
                System.err.println("Error occured while opening Settings: " + 
                        ex.toString());
            }
        }
        if(m_singleton==null) {
            String currencySymbol = m_settings.getStringProperty("currency","");
            int entryType = m_settings.getIntProperty("entry-type", 0);
            m_singleton = new CarManagerSettings(currencySymbol, entryType);
        }
        return m_singleton;
    }
    
    /** Get currency symbol */
    public String getCurrencySymbol() {
        return m_currencySymbol;
    }
    
    /** Set currency symbol */
    public void setCurrencySymbol(String symbol) {
        m_currencySymbol = symbol;
        m_settings.setStringProperty("currency", symbol);
        try {
            m_settings.save(true);
        }catch(Exception ex) {
            System.err.println("Error while saving settings: " + ex.toString());
        }
    }
    
    /** Get entry type */
    public int getEntryType() {
        return m_entryType;
    }
    
    /** Set entry type */
    public void setEntryType(int type) {
        m_entryType = type;
        m_settings.setIntProperty("entry-type", type);
        try{
            m_settings.save(true);
        }catch(Exception ex) {
            System.err.println("Error while saving settings: " + ex.toString());
        }
    }
    
}
