package com.substanceofcode.carmanager.view;

import com.substanceofcode.carmanager.controller.CarManagerMIDlet;
import com.substanceofcode.carmanager.model.FillupEntryParser;
import java.util.Calendar;
import java.util.Date;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.DateField;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.TextField;
import com.substanceofcode.carmanager.model.FillupEntry;


public class EntryForm extends Form implements CommandListener {
    private final CarManagerMIDlet carManagerMIDlet;
    
    private String m_oldRecord;
    private DateField gasDate = new DateField("Date", DateField.DATE);
    private TextField gasLitres = new TextField("Fill-up in centilitres", "", 10, TextField.NUMERIC);
    private TextField totalPrice = new TextField("Total price (in cents)", "", 10, TextField.NUMERIC);
    private TextField pricePerLitre = new TextField("Cost per liter (in 1/10th cents)", "", 10, TextField.NUMERIC);
    private TextField kilometers = new TextField("Current mileage", "", 10, TextField.NUMERIC);
    private TextField comment = new TextField("Comment", "", 32, TextField.ANY);
    private String[] stringArray = {"Full fill-up"};
    
    // create the list of choice groups.
    ChoiceGroup group = new ChoiceGroup("Type", ChoiceGroup.MULTIPLE, stringArray, null);
    
    Command cmdSave = new Command("Save", Command.SCREEN, 1);
    Command cmdCancel = new Command("Cancel", Command.SCREEN, 2);
    
    private int entryType = 0;
    private final static int ENTRYTYPE_TOTALFILLUP = 0;
    private final static int ENTRYTYPE_PRICEPERLITER = 1;
    
    public EntryForm(CarManagerMIDlet carManagerMIDlet, int entryType) {
        super(null);
        this.carManagerMIDlet = carManagerMIDlet;
        m_oldRecord = new String();
        Calendar c = Calendar.getInstance();
        gasDate.setDate(c.getTime());
        append(gasDate);

        this.entryType = entryType;
        if(this.entryType == ENTRYTYPE_TOTALFILLUP) {
            /** User wants to enter total fillup */
            append(gasLitres);
        } else {
            /** User wants to enter price per liter */
            append(pricePerLitre);
        }
        append(group);
        append(totalPrice);

        append(kilometers);
        append(comment);
        addCommand(cmdSave);
        addCommand(cmdCancel);
        setCommandListener(this);
    }
    
    public void clearView(long kilometersValue) {
        m_oldRecord = "";
        Calendar c = Calendar.getInstance();
        gasDate.setDate(c.getTime());
        gasLitres.setString( "" );
        group.setSelectedIndex(0,false);
        totalPrice.setString( "" );
        kilometers.setString( Long.toString(kilometersValue)  );
        comment.setString( "" );
    }
    
    public void setView(FillupEntry entry) {
        m_oldRecord = entry.getOriginalRecord();
        String date = entry.getDate();
        Calendar c = Calendar.getInstance();
        c.set(Calendar.YEAR, Integer.parseInt(date.substring(0,4)));
        c.set(Calendar.MONTH, Integer.parseInt(date.substring(4,6)) - 1);
        c.set(Calendar.DAY_OF_MONTH, Integer.parseInt(date.substring(6,8)));
        gasDate.setDate(c.getTime());
        gasLitres.setString( String.valueOf(entry.getGas()) );
        pricePerLitre.setString( String.valueOf(entry.getPricePerLiter()) );
        
        /** Full fill-up */
        if( entry.getFullFillUp()!=0 ) {
            group.setSelectedIndex(0,true);
        }
        else {
            group.setSelectedIndex(0,false);
        }
        
        totalPrice.setString( String.valueOf(entry.getPrice())  );
        kilometers.setString( String.valueOf(entry.getKm())  );
        comment.setString( entry.getComment() );
    }
    
    /** Handle entry form's commands */
    public void commandAction(Command c, Displayable s) {
        if( c==cmdCancel ) {
            carManagerMIDlet.showMainCanvas();
            return;
        }
        if (c == cmdSave) {
            Date date = gasDate.getDate();
            Calendar cal = Calendar.getInstance();
            try {
                cal.setTime(date);
            } catch (Exception e) {
                System.err.println("setTime: " + e.toString());
            }
            String month = String.valueOf(cal.get(java.util.Calendar.MONTH) + 1);
            if( month.length()<2 ) month = "0" + month;
            String day = String.valueOf(cal.get(java.util.Calendar.DATE));
            if( day.length()<2 ) day = "0" + day;
            String year = String.valueOf(cal.get(java.util.Calendar.YEAR));
            if( year.length()<4 ) year = "0" + year;
            
            int intFullFillUp = 0;
            if( group.isSelected(0) )
                intFullFillUp = 1;

            /** Total price (in cents) */
            String strPrice = totalPrice.getString();
            if( strPrice.length()==0) {
                strPrice="0";
            }
            int totalPriceValue;
            try{
                totalPriceValue = Integer.parseInt(strPrice);
            }catch(Exception ex){
                totalPriceValue = 0;
            }
            
            /** 
             * Entry style defines that either price per liter or total
             * gas liters is calculated based on opposite value.
             */
            String strGasLitres;
            if(this.entryType == ENTRYTYPE_TOTALFILLUP) {
                /** Total gas centilitres is given */
                strGasLitres = gasLitres.getString();
                if( strGasLitres.length()==0) strGasLitres="0";
                
                /** 
                 * Price per liter (in cents) is calculated as:
                 * (TOTALPRICE*1000)/TOTALGAS = (1/1000)/l
                 */
                int gasLitresValue;
                try{
                    gasLitresValue = Integer.parseInt( strGasLitres );
                } catch(Exception ex) {
                    gasLitresValue = 0;
                }
                
                /** This value is never used ... */
                int pricePerLitreValue = (gasLitresValue*1000)/totalPriceValue;
                
            } else {
                /** Price per liter is given */
                String strPricePerLiter = pricePerLitre.getString();
                if( strPricePerLiter.length()==0) {
                    strPricePerLiter="0";
                }
                int pricePerLiterValue;
                try{
                    pricePerLiterValue = Integer.parseInt(strPricePerLiter);
                }catch(Exception ex){
                    pricePerLiterValue = 0;
                }
                
                /** 
                 * Total gas is calculated as:
                 * (TOTALPRICE*1000)/PRICEPERLITER = 
                 */                
                int gasLitresValue;
                gasLitresValue = (totalPriceValue*1000)/pricePerLiterValue;
                strGasLitres = String.valueOf( gasLitresValue );
                
            }
            
            /** Current mileage */
            String strTrip = kilometers.getString();
            if( strTrip.length()==0) strTrip="0";
            
            String newRecord;
            newRecord = year + month + day;
            newRecord = newRecord + "|" + strGasLitres;
            newRecord = newRecord + "|" + String.valueOf(intFullFillUp);
            newRecord = newRecord + "|" + strPrice;
            newRecord = newRecord + "|" + strTrip;
            newRecord = newRecord + "|" + comment.getString();
            System.out.println("Data: " + newRecord);
            
            FillupEntry tank = FillupEntryParser.parseEntry( newRecord );

            if(m_oldRecord.length()>0) {
                // Update old entry
                carManagerMIDlet.updateFillUp(tank, newRecord, m_oldRecord);
            } else {
                // Create new entry
                carManagerMIDlet.addFillUp(tank, newRecord);
            }
            carManagerMIDlet.showMainCanvas();
        }
    }
}